#ifndef THREAD_H
#define THREAD_H

#ifdef NT_PTHREADS

#include <windows.h> // for handle
#include <process.h> // for _beginthreadex and _endthreadex


/* thread scope - who is in scheduling pool */
#define PTHREAD_SCOPE_PROCESS   0
#define PTHREAD_SCOPE_SYSTEM    0

/* default attr states */
#define pthread_mutexattr_default	NULL
#define pthread_condattr_default	NULL

/* thread state - joinable or not */
#define PTHREAD_CREATE_JOINABLE 0
#define PTHREAD_CREATE_DETACHED 0

/* mutex and condition variable scope - process or system */
#define PTHREAD_SHARE_PRIVATE   0
#define PTHREAD_SHARE_PROCESS   0

/* thread attributes and thread type */
typedef int	pthread_attr_t;
typedef HANDLE	pthread_t;

/* mutex attributes and mutex type */
typedef int	pthread_mutexattr_t;
typedef HANDLE	pthread_mutex_t;


/* condition variable attributes and condition variable type */
typedef int pthread_condattr_t;
typedef HANDLE pthread_cond_t;


int		pthread_attr_init( pthread_attr_t *attr );
int		pthread_attr_destroy( pthread_attr_t *attr );
int		pthread_attr_getdetachstate( pthread_attr_t *attr, int *detachstate );
int		pthread_attr_setdetachstate( pthread_attr_t *attr, int detachstate );
int		pthread_create(pthread_t *tid, pthread_attr_t attr, LPTHREAD_START_ROUTINE func, LPVOID arg);


#define pthread_yield();
void	pthread_exit();
#define pthread_kill(a, b);
void	pthread_join( pthread_t tid, int *status );
int		pthread_mutex_init( pthread_mutex_t *mp, pthread_mutexattr_t *attr );
int		pthread_mutex_destroy( pthread_mutex_t *mp );
int		pthread_mutex_lock( pthread_mutex_t *mp );
int		pthread_mutex_unlock( pthread_mutex_t *mp );
int		pthread_mutex_trylock( pthread_mutex_t *mp );
int		pthread_cond_init( pthread_cond_t *cv, pthread_condattr_t *attr );
int		pthread_cond_destroy( pthread_cond_t *cv );
int		pthread_cond_wait( pthread_cond_t *cv, pthread_mutex_t *mp );
int		pthread_cond_signal( pthread_cond_t *cv );

#endif /* nt_pthreads */
#endif 
