/* Copyright (c) 1997 QUALCOMM Incorporated.
 * All rights reserved.
 */
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include "slap.h"
#include "back-ldbm.h"

void ldif2id2entry(char *, Backend *);
void ldif2id2children(char *, Backend *);
void indexattributes(char *, Backend *);
void ldif2index(char *attr, char *inputfile, Backend *be);


extern void		attr_index_config();
extern char		*str_getline();
extern int		nbackends;
extern Backend		*backends;

typedef struct tagNPAIRS
{
	char *attributeName;
	char *replacewith;
} NPAIRS ;

void normalizeAttributes(char *infile, char *outfile, NPAIRS *normals)
{
	char		*linep, *buf;
	char		line[BUFSIZ];
	int			stop, lineno, elineno, i;
	int      	lmax, lcur;
	struct berval	bv;
	struct berval	*vals[2];
	Avlnode		*avltypes = NULL;
	FILE *ofile, *ifile;

	if((ofile = fopen(outfile,"w")) == NULL)
	{
		perror(outfile);
		exit( 1 );
	}
	if((ifile = fopen(infile, "r")) == NULL)
	{
		perror(infile);
		exit(1);
	}

	stop = 0;
	buf = NULL;
	lineno = 0;
	lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;
	while ( ! stop ) 
	{
		char	*type, *val, *s;
		int		vlen;

		if ( fgets( line, sizeof(line), ifile ) != NULL ) 
		{
			int     len;

			lineno++;
			len = strlen( line );
			while ( lcur + len + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;
		if ( line[0] == '\n' || stop && buf && *buf ) /* Read an entry from ldif file */ 
		{
			s = buf;
			elineno = 0;
			while ( (linep = str_getline( &s )) != NULL ) 
			{
				elineno++;
				if ( str_parse_line( linep, &type, &val, &vlen ) != 0 ) 
				{
					Debug( LDAP_DEBUG_PARSE,"bad line %d in entry ending at line %d ignored\n", elineno, lineno, 0 );
					continue;
				}
				attr_normalize(type);
				for( i=0;normals[i].attributeName;i++)
					if(strcmp(type,normals[i].attributeName) == 0)
						type = normals[i].replacewith;

				fputs(type,ofile);
				fputs(": ",ofile);
				fputs(val,ofile);
				fputs("\n",ofile);
				if ( !isascii( *type ) || isdigit( *type ) )
					continue;
				type = strdup( type );
			}
			fputs("\n",ofile);
			*buf = '\0';
			lcur = 0;
		}
	}

	fclose(ifile);
	fclose(ofile);
}

void ldif2id2entry(char *inputfile, Backend *be)
{
	int			stop, lmax, lcur;
	char		line[BUFSIZ], idbuf[BUFSIZ], *buf;
	ID			id;
	struct dbcache	*db;
	struct berval	bv, *vals[2];
	Avlnode		*avltypes = NULL;
	FILE		*fp;

	if ( (db = ldbm_cache_open( be, "id2entry", LDBM_SUFFIX, LDBM_NEWDB )) == NULL ) 
	{
		perror( "id2entry file" );
		exit( 1 );
	}

	if ((fp =  fopen( inputfile, "r" )) == NULL ) 
	{
		perror( inputfile );
		exit( 1 );
	}
	id = stop = lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;
	buf = NULL;
	while ( ! stop ) 
	{
		Datum		key, data;

		if ( fgets( line, sizeof(line), fp ) != NULL ) 
		{
			int     len, idlen;

			len = strlen( line );
			if ( buf == NULL || *buf == '\0' ) 
			{
				sprintf( idbuf, "%d\n", id + 1 );
				idlen = strlen( idbuf );
			} 
			else 
				idlen = 0;

			while ( lcur + len + idlen + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}

			if ( idlen > 0 ) 
			{
				strcpy( buf + lcur, idbuf );
				lcur += idlen;
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;
		if ( line[0] == '\n' || stop && buf && *buf ) 
		{
			if ( *buf != '\n' ) 
			{
				id++;
				key.dptr = (char *) &id;
				key.dsize = sizeof(ID);
				data.dptr = buf;
				data.dsize = strlen( buf ) + 1;
				if ( ldbm_store( db->dbc_db, key, data, LDBM_INSERT ) != 0 ) 
				{
					perror( "id2entry ldbm_store" );
					exit( 1 );
				}
			}
			*buf = '\0';
			lcur = 0;
			line[0] = '\0';
		}
	}
	(*be->be_close)( be );
	fclose(fp);

	id++;
	sprintf( line, "%s/NEXTID", ((struct ldbminfo *) be->be_private)->li_directory );
	if ( (fp = fopen( line, "w" )) == NULL ) 
	{
		perror( line );
		fprintf( stderr, "Could not write next id %ld\n", id );
		exit(1);
	} 
	fprintf( fp, "%ld\n", id );
	fclose( fp );
	return;
}

void ldif2id2children(char *inputfile, Backend *be)
{
	char		*linep, *buf, line[BUFSIZ];
	int			stop, lmax, lcur, lineno, elineno;
	ID		id;
	struct dbcache	*db, *db2;
	struct berval	bv, *vals[2];
	Avlnode		*avltypes = NULL;
	FILE *fp;

	if ((fp =  fopen( inputfile, "r" )) == NULL ) 
	{
		perror( inputfile );
		exit( 1 );
	}

	/*
	 * first, make the dn2id index
	 */

	if ( (db = ldbm_cache_open( be, "dn2id", LDBM_SUFFIX, LDBM_NEWDB )) == NULL ) 
	{
		perror( "dn2id file" );
		exit( 1 );
	}

	buf = NULL;
	id = stop = lineno = lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;

	while ( ! stop ) 
	{
		char		*type, *val, *s;
		int			vlen;
		Datum		key, data;

		if ( fgets( line, sizeof(line), fp ) != NULL ) 
		{
			int     len;

			lineno++;
			len = strlen( line );
			while ( lcur + len + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;

		if ( line[0] == '\n' || stop && buf && *buf ) /* After a complete entry was read into buf */
		{
			if ( *buf != '\n' ) 
			{
				id++;
				s = buf;
				elineno = 0; /*line number inside entry */ 
				while ( (linep = str_getline( &s )) != NULL ) /* Read a line from entry */ 
				{
					elineno++;
					if ( str_parse_line( linep, &type, &val, &vlen ) != 0 ) /* parses type: val format, val is null terminated */
					{
						Debug( LDAP_DEBUG_PARSE, "bad line %d in entry ending at line %d ignored\n", elineno, lineno, 0 );
						continue;
					}
					if ( strcmp( type, "dn" ) == 0 ) /* encountered dn:<> */
						break;
				}
				if ( linep == NULL ) 
					fprintf( stderr, "entry %d has no dn\n", id );
				else 
				{
					key.dptr = dn_normalize_case( val ); 
					key.dsize = strlen( val ) + 1;
					data.dptr = (char *) &id;
					data.dsize = sizeof(ID);
					if ( ldbm_store( db->dbc_db, key, data, LDBM_REPLACE ) != 0 ) 
					{
						perror( "dn2id ldbm_store" );
						exit( 1 );
					}
				}
			}
			*buf = '\0';
			lcur = 0;
			line[0] = '\0';
		}
	}

	/*
	 * next, make the id2children index
	 */

	if ( (db2 = ldbm_cache_open( be, "id2children", LDBM_SUFFIX, LDBM_NEWDB )) == NULL ) 
	{
		perror( "id2children file" );
		exit( 1 );
	}

	rewind( fp ); /* start from beginning of input file */
	id = 0;
	stop = 0;
	buf = NULL;
	lineno = 0;
	lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;
	while ( ! stop ) 
	{
		char	*type, *val, *s, *dn;
		int	vlen;
		ID	pid;
		char	buf2[20];
		Datum	key, data;

		if ( fgets( line, sizeof(line), fp ) != NULL ) 
		{
			int     len;

			len = strlen( line );
			while ( lcur + len + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;
		if ( line[0] == '\n' || stop && buf && *buf )  /* Read an entry */
		{
			if ( * buf != '\n' ) 
			{
				id++;
				s = buf;
				while ( (linep = str_getline( &s )) != NULL ) 
				{
					if ( str_parse_line( linep, &type, &val, &vlen ) != 0 ) 
					{
						Debug( LDAP_DEBUG_PARSE, "bad line %d ignored\n", lineno, 0, 0 );
						continue;
					}

					if ( strcmp( type, "dn" ) == 0 )
						break;
				}

				if ( linep == NULL ) 
					fprintf( stderr, "entry %d has no dn\n", id );
				else 
				{
					if ( (dn = dn_parent( be, val )) == NULL ) 
						pid = 0;
					else 
					{
						key.dptr = dn_normalize_case( dn );
						key.dsize = strlen( dn ) + 1;

						data = ldbm_fetch( db->dbc_db, key );
						if ( data.dptr == NULL ) 
						{
							dn_normalize( val );
							if ( ! be_issuffix( be, val ) ) 
								Debug( LDAP_DEBUG_PARSE, "no parent \"%s\" of \"%s\"\n", dn, val, 0 );
							*buf = '\0';
							lcur = 0;
							line[0] = '\0';
							continue;
						}
						(void) memcpy( (char *) &pid, data.dptr, sizeof(ID) );
					}

					sprintf( buf2, "%c%d", EQ_PREFIX, pid );
					key.dptr = buf2;
					key.dsize = strlen( buf2 ) + 1;
					if ( idl_insert_key( be, db2, key, id ) != 0 ) 
					{
						perror( "idl_insert_key" );
						exit( 1 );
					}
				}
			}
			*buf = '\0';
			lcur = 0;
			line[0] = '\0';
		}
	}
	fclose(fp);
	(*be->be_close)( be );
}

void indexattributes(char *inputfile, Backend *be)
{
	int			stop;
	char		*linep, *buf;
	char		line[BUFSIZ];
	int			lineno, elineno;
	int      	lmax, lcur;
	ID		id;
	struct berval	bv;
	struct berval	*vals[2];
	Avlnode		*avltypes = NULL;
	FILE *fp;

	if ((fp =  fopen( inputfile, "r" )) == NULL ) 
	{
		perror( inputfile );
		exit( 1 );
	}


	id = 0;
	stop = 0;
	buf = NULL;
	lineno = 0;
	lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;
	while ( ! stop ) {
		char	*type, *val, *s;
		int		vlen, indexmask, syntaxmask;

		if ( fgets( line, sizeof(line), fp ) != NULL ) 
		{
			int     len;

			lineno++;
			len = strlen( line );
			while ( lcur + len + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;
		if ( line[0] == '\n' || stop && buf && *buf ) /* Read an entry from ldif file */ 
		{
			id++;
			s = buf;
			elineno = 0;
			while ( (linep = str_getline( &s )) != NULL ) 
			{
				elineno++;
				if ( str_parse_line( linep, &type, &val, &vlen ) != 0 ) 
				{
					Debug( LDAP_DEBUG_PARSE,"bad line %d in entry ending at line %d ignored\n", elineno, lineno, 0 );
					continue;
				}

				if ( !isascii( *type ) || isdigit( *type ) )
					continue;

				type = strdup( type );
				if ( avl_insert( &avltypes, type, stricmp, avl_dup_error ) != 0 ) /* Make a tree from the attribute type */  
					free( type );
				else 
				{
					attr_masks( be->be_private, type, &indexmask, &syntaxmask );
					if ( indexmask ) 
						ldif2index(type, inputfile, be);
				}
			}
			*buf = '\0';
			lcur = 0;
		}
	}
	(*be->be_close)( be );

	fclose(fp);
}

void ldif2index(char *attr, char *inputfile, Backend *be)
{
	char		*linep, *buf;
	char		line[BUFSIZ];
	int			stop, lmax, lcur, lineno, elineno;
	unsigned long	id;
	struct berval	bv, *vals[2];
	FILE *fp;

	if ((fp =  fopen( inputfile, "r" )) == NULL ) 
	{
		perror( inputfile );
		exit( 1 );
	}

	id = 0;
	stop = 0;
	lineno = 0;
	buf = NULL;
	lcur = lmax = 0;
	vals[0] = &bv;
	vals[1] = NULL;
	while ( ! stop ) {
		char		*type, *val, *s;
		int		vlen;

		if ( fgets( line, sizeof(line), fp ) != NULL ) 
		{
			int     len;

			lineno++;
			len = strlen( line );
			while ( lcur + len + 1 > lmax ) 
			{
				lmax += BUFSIZ;
				buf = (char *) ch_realloc( buf, lmax );
			}
			strcpy( buf + lcur, line );
			lcur += len;
		} 
		else 
			stop = 1;
		if ( line[0] == '\n' || stop && buf && *buf ) 
		{
			if ( *buf != '\n' ) 
			{
				id++;
				s = buf;
				elineno = 0;
				while ( (linep = str_getline( &s )) != NULL ) 
				{
					elineno++;
					if ( str_parse_line( linep, &type, &val, &vlen ) != 0 ) 
					{
						Debug( LDAP_DEBUG_PARSE, "bad line %d in entry ending at line %d ignored\n", elineno, elineno, 0 );
						continue;
					}

					if ( strcasecmp( type, attr ) == 0 ) 
					{
						bv.bv_val = val;
						bv.bv_len = vlen;
						index_add_values( be, attr, vals, id );
					}
				}
			}
			*buf = '\0';
			lcur = 0;
		}
	}
	(*be->be_close)( be );
	fclose(fp);
}


