/* lock.c - routines to open and apply an advisory lock to a file */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include "ntport.h"
#include "portable.h"
#include <io.h>
#include <sys/locking.h>
#include <winsock.h>
#include "slap.h"

#define LOCK_EX _LK_NBLCK
#define LOCK_UN _LK_UNLCK
#define flock(a, b) locking(a,b,1)

FILE *
lock_fopen( char *fname, char *type, FILE **lfp )
{
	FILE	*fp;
	char	buf[MAX_PATH];

	/* open the lock file */
	strcpy( buf, fname );
	strcat( buf, ".lock" );
	if ( (*lfp = fopen( buf, "w" )) == NULL ) {
		Debug( LDAP_DEBUG_ANY, "could not open \"%s\"\n", buf, 0, 0 );
		return( NULL );
	}

	while ( flock( fileno( *lfp ), LOCK_EX ) != 0 ) ; /* Try until lock is obtained */

	/* open the log file */
	if ( (fp = fopen( fname, type )) == NULL ) 
	{
		Debug( LDAP_DEBUG_ANY, "could not open \"%s\"\n", fname, 0, 0 );
		flock( fileno( *lfp ), LOCK_UN ); /* Unlock the lock file */
		return( NULL );
	}

	return( fp );
}

int
lock_fclose( FILE *fp, FILE *lfp )
{
	flock( fileno( lfp ), LOCK_UN ); /* Unlock the lock file */
	fclose( lfp );

	return( fclose( fp ) );
}
