/* Copyright (c) 1997 QUALCOMM Incorporated.
 * All rights reserved.
 */
#ifdef WINAPP
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "slap.h"
#include "back-ldbm.h"

void init(void);
void ldif2id2entry(char *, Backend *);
void ldif2id2children(char *, Backend *);
void indexattributes(char *, Backend *);
void ldif2index(char *attr, char *inputfile, Backend *be);

int		dummyClSock;
struct sockaddr_in dummyaddr;

extern void		attr_index_config();
extern char		*str_getline();
extern int		nbackends;
extern Backend		*backends;

int		ldap_debug;
int		ldap_syslog;
int		ldap_syslog_level;
int		global_schemacheck;
int		num_entries_sent;
int		num_bytes_sent;
int		active_threads;
char		*default_referral;
struct objclass	*global_oc;
time_t		currenttime;
pthread_t	listener_tid;
pthread_mutex_t	num_sent_mutex;
pthread_mutex_t	entry2str_mutex;
pthread_mutex_t	active_threads_mutex;
pthread_mutex_t	new_conn_mutex;
pthread_mutex_t	currenttime_mutex;
pthread_mutex_t	replog_mutex;
pthread_mutex_t	ops_mutex;
pthread_mutex_t	regex_mutex;


static char	*indexcmd;
static char	*tailorfile;
static char	*inputfile;
static int      maxkids = 1;
static int      nkids;
 
static void
usage( char *name )
{
	char temp[512];
	//fprintf( stderr, "usage: %s <inputfile> <configfile>\n", name );
	sprintf( temp, "usage: %s <inputfile> <configfile>\n", name );
	MessageBox(NULL, temp, __argv[0], MB_OK);

	exit( 1 );
}



int GetWorkDir(char *WorkDir){

	char buf[512];
	GetModuleFileName(NULL, buf, 512);
	if ( GetModuleFileName( NULL, buf, 512 ) != 0 )
	{
		//trim the file name
		char *ptr = buf + strlen(buf);
		while(*ptr != '\\')
			ptr--;
		*(++ptr) = '\0' ;
		
		strcpy(WorkDir, buf);
		if(WorkDir[strlen(WorkDir)-1] != '\\')
		strcat(WorkDir, "\\");
	}
	return 0;
}



#ifdef WINAPP
int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmdLine, int nCmdShow)
#else
void main( int argc, char **argv )
#endif

{
	int			i;
	Backend		*be = NULL;
	Avlnode		*avltypes = NULL;
	FILE *fp;
	char PathName[512];


	if (__argc != 3)
		usage(__argv[0]);

	inputfile  = strdup(__argv[1]);
	tailorfile = strdup(__argv[2]); 

	
	if ((fp =  fopen( inputfile, "r" )) == NULL ) 
	{
		perror( inputfile );
		exit( 1 );
	}
	
	fclose(fp);
	
	
	// Set the working directory to that of IDS directory
	if(  GetWorkDir(PathName))
		SetCurrentDirectory(PathName);
	
	
	/*
	 * initialize stuff and figure out which backend we're dealing with
	 */

	init();
	read_config( tailorfile, &be, NULL );

	/*
	 * Utility only supports ldbm database type
	 */

	for ( i = 0; i < nbackends; i++ ) 
		if ( strcasecmp( backends[i].be_type, "ldbm" ) == 0 ) 
				break;
	if ( i == nbackends )  
	{
		char temp[512];
		//fprintf( stderr, "No ldbm database found in config file\n" );
		strcpy( temp, "No ldbm database found in config file" );
		MessageBox(NULL, temp, __argv[0], MB_OK);
		exit( 1 );
	}
	be = &backends[i];

	/*
	 * generate the id2entry index
	 */

	ldif2id2entry(inputfile, be);

	/*
	 * generate the dn2id and id2children indexes
	 */

	ldif2id2children(inputfile, be);


	/*
	 * generate the attribute indexes
	 */

	indexattributes(inputfile, be);

	exit( 0 );
}


void init()
{
	pthread_mutex_init( &active_threads_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &new_conn_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &currenttime_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &entry2str_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &replog_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &ops_mutex, pthread_mutexattr_default );
	pthread_mutex_init( &num_sent_mutex, pthread_mutexattr_default );
#ifndef sunos5
	pthread_mutex_init( &regex_mutex, pthread_mutexattr_default );
#endif
}

#include <stdarg.h>

void Debug(int level, char* fmt, ...)
{

}